@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2012 Marc Alexander Lehmann <schmorp@schmorp.de>
+ * Copyright (c) 2001-2012,2015 Marc Alexander Lehmann <schmorp@schmorp.de>
  *
  * Redistribution and use in source and binary forms, with or without modifica-
  * tion, are permitted provided that the following conditions are met:
@@ -78,6 +78,20 @@
  * 2012-12-04 reduce misprediction penalty for x86/amd64 assembly switcher.
  * 2012-12-05 experimental fiber backend (allocates stack twice).
  * 2012-12-07 API version 3 - add coro_stack_alloc/coro_stack_free.
+ * 2012-12-21 valgrind stack registering was broken.
+ * 2015-12-05 experimental asm be for arm7, based on a patch by Nick Zavaritsky.
+ *            use __name__ for predefined symbols, as in libecb.
+ *            enable guard pages on arm, aarch64 and mips.
+ * 2016-08-27 try to disable _FORTIFY_SOURCE with CORO_SJLJ, as it
+ *            breaks setjmp/longjmp. Also disable CORO_ASM for asm by default,
+ *            as it was reported to crash.
+ * 2016-11-18 disable cfi_undefined again - backtraces might be worse, but
+ *            compile compatibility is improved.
+ * 2018-08-14 use a completely different pthread strategy that should allow
+ *            sharing of coroutines among different threads. this would
+ *            undefined behaviour before as mutexes would be unlocked on
+ *            a different thread. overall, this might be slower than
+ *            using a pipe for synchronisation, but pipes eat fd's...
  */
 
 #ifndef CORO_H
@@ -139,8 +153,8 @@ extern "C" {
  * -DCORO_ASM
  *
  *    Hand coded assembly, known to work only on a few architectures/ABI:
- *    GCC + x86/IA32 and amd64/x86_64 + GNU/Linux and a few BSDs. Fastest choice,
- *    if it works.
+ *    GCC + arm7/x86/IA32/amd64/x86_64 + GNU/Linux and a few BSDs. Fastest
+ *    choice, if it works.
  *
  * -DCORO_PTHREAD
  *
@@ -298,11 +312,11 @@ void coro_stack_free (struct coro_stack *stack);
     && !defined CORO_SJLJ    && !defined CORO_LINUX \
     && !defined CORO_IRIX    && !defined CORO_ASM \
     && !defined CORO_PTHREAD && !defined CORO_FIBER
-# if defined WINDOWS && (defined __x86 || defined __amd64 || defined _M_IX86 || defined _M_AMD64)
+# if defined WINDOWS && (defined __i386__ || (__x86_64__ || defined _M_IX86 || defined _M_AMD64)
 #  define CORO_ASM 1
 # elif defined WINDOWS || defined _WIN32
 #  define CORO_LOSER 1 /* you don't win with windoze */
-# elif defined __linux && (defined __x86 || defined __amd64)
+# elif __linux && (__i386__ || (__x86_64__ && !__ILP32__) /*|| (__arm__ && __ARM_ARCH == 7)), not working */
 #  define CORO_ASM 1
 # elif defined HAVE_UCONTEXT_H
 #  define CORO_UCONTEXT 1
@@ -333,6 +347,12 @@ struct coro_context
 #  define _GNU_SOURCE /* for glibc */
 # endif
 
+/* try to disable well-meant but buggy checks in some libcs */
+# ifdef _FORTIFY_SOURCE
+#  undef _FORTIFY_SOURCE
+#  undef __USE_FORTIFY_LEVEL /* helps some more when too much has been included already */
+# endif
+
 # if !CORO_LOSER
 #  include <unistd.h>
 # endif
@@ -374,7 +394,11 @@ struct coro_context
   void **sp; /* must be at offset 0 */
 };
 
+#if __i386__ || __x86_64__
 void __attribute__ ((__noinline__, __regparm__(2)))
+#else
+void __attribute__ ((__noinline__))
+#endif
 coro_transfer (coro_context *prev, coro_context *next);
 
 # define coro_destroy(ctx) (void *)(ctx)
@@ -387,8 +411,8 @@ extern pthread_mutex_t coro_mutex;
 
 struct coro_context
 {
+  int flags;
   pthread_cond_t cv;
-  pthread_t id;
 };
 
 void coro_transfer (coro_context *prev, coro_context *next);
